/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 Module:       MainTest.cs
 Author:       Lorne McIntosh
 Date started: 2007/07/31
 Purpose:      Example implementation for JSBSimNET

--------- Copyright (C) 2007  Lorne D. McIntosh (lorne@lorneswork.com) ---------

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

HISTORY
--------------------------------------------------------------------------------
2007/07/31   LDM   Created

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

namespace JSBSimTest
{
	using System;
	using System.Drawing;
	using System.Collections;
	using System.ComponentModel;
	using System.Windows.Forms;
	using System.IO;
    using System.Diagnostics;

	/// <summary>
	/// Test interface for JSBSimCSharp
	/// </summary>
    public class MainTest : Form
    {
        private MenuStrip menuStrip1;
        private ToolStripMenuItem ToolStripMenuItem_Load;
        private ToolStripMenuItem ToolStripMenuItem_LoadScripts;
        private ToolStripMenuItem ToolStripMenuItem_LoadModels;
        private StatusStrip statusStrip1;
        private ToolStripMenuItem ToolStripMenuItem_Cancel;
        private ToolStripStatusLabel toolStripStatusLabel1;
        private IContainer components;
        
        // Create an instance of our jsbsim worker thread
        private JSBSimWorker jsbsimworker;
        private TextBox tb_console;

        // Define the root directory where the aircraft, engine, and scripts subdirectories are located
        private string rootDirectory = "..\\..\\..\\JSBSim";

        public MainTest()
        {
            InitializeComponent();

            Trace.Listeners.Clear();
            Trace.Listeners.Add(new TextBoxTextWriterTraceListener(tb_console));
            //Trace.Listeners.Add(new TextWriterTraceListener(Console.Out));

            jsbsimworker = new JSBSimWorker(rootDirectory);

            //attach an event handler to be called when the worker completes
            jsbsimworker.RunWorkerCompleted +=
                new RunWorkerCompletedEventHandler(WorkerCompleted);
        }

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            Application.Run(new MainTest());
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.menuStrip1 = new System.Windows.Forms.MenuStrip();
            this.ToolStripMenuItem_Load = new System.Windows.Forms.ToolStripMenuItem();
            this.ToolStripMenuItem_LoadScripts = new System.Windows.Forms.ToolStripMenuItem();
            this.ToolStripMenuItem_LoadModels = new System.Windows.Forms.ToolStripMenuItem();
            this.ToolStripMenuItem_Cancel = new System.Windows.Forms.ToolStripMenuItem();
            this.statusStrip1 = new System.Windows.Forms.StatusStrip();
            this.toolStripStatusLabel1 = new System.Windows.Forms.ToolStripStatusLabel();
            this.tb_console = new System.Windows.Forms.TextBox();
            this.menuStrip1.SuspendLayout();
            this.statusStrip1.SuspendLayout();
            this.SuspendLayout();
            // 
            // menuStrip1
            // 
            this.menuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.ToolStripMenuItem_Load,
            this.ToolStripMenuItem_Cancel});
            this.menuStrip1.Location = new System.Drawing.Point(0, 0);
            this.menuStrip1.Name = "menuStrip1";
            this.menuStrip1.Size = new System.Drawing.Size(632, 24);
            this.menuStrip1.TabIndex = 2;
            this.menuStrip1.Text = "menuStrip1";
            // 
            // ToolStripMenuItem_Load
            // 
            this.ToolStripMenuItem_Load.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.ToolStripMenuItem_LoadScripts,
            this.ToolStripMenuItem_LoadModels});
            this.ToolStripMenuItem_Load.Name = "ToolStripMenuItem_Load";
            this.ToolStripMenuItem_Load.Size = new System.Drawing.Size(42, 20);
            this.ToolStripMenuItem_Load.Text = "Load";
            this.ToolStripMenuItem_Load.DropDownOpening += new System.EventHandler(this.ToolStripMenuItem_Load_DropDownOpening);
            // 
            // ToolStripMenuItem_LoadScripts
            // 
            this.ToolStripMenuItem_LoadScripts.Name = "ToolStripMenuItem_LoadScripts";
            this.ToolStripMenuItem_LoadScripts.Size = new System.Drawing.Size(107, 22);
            this.ToolStripMenuItem_LoadScripts.Text = "Scripts";
            // 
            // ToolStripMenuItem_LoadModels
            // 
            this.ToolStripMenuItem_LoadModels.Name = "ToolStripMenuItem_LoadModels";
            this.ToolStripMenuItem_LoadModels.Size = new System.Drawing.Size(107, 22);
            this.ToolStripMenuItem_LoadModels.Text = "Models";
            // 
            // ToolStripMenuItem_Cancel
            // 
            this.ToolStripMenuItem_Cancel.Enabled = false;
            this.ToolStripMenuItem_Cancel.Name = "ToolStripMenuItem_Cancel";
            this.ToolStripMenuItem_Cancel.Size = new System.Drawing.Size(51, 20);
            this.ToolStripMenuItem_Cancel.Text = "Cancel";
            this.ToolStripMenuItem_Cancel.Click += new System.EventHandler(this.cancelToolStripMenuItem_Click);
            // 
            // statusStrip1
            // 
            this.statusStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.toolStripStatusLabel1});
            this.statusStrip1.Location = new System.Drawing.Point(0, 424);
            this.statusStrip1.Name = "statusStrip1";
            this.statusStrip1.Size = new System.Drawing.Size(632, 22);
            this.statusStrip1.TabIndex = 4;
            this.statusStrip1.Text = "statusStrip1";
            // 
            // toolStripStatusLabel1
            // 
            this.toolStripStatusLabel1.Name = "toolStripStatusLabel1";
            this.toolStripStatusLabel1.Size = new System.Drawing.Size(38, 17);
            this.toolStripStatusLabel1.Text = "Ready";
            // 
            // tb_console
            // 
            this.tb_console.BackColor = System.Drawing.SystemColors.Window;
            this.tb_console.Dock = System.Windows.Forms.DockStyle.Fill;
            this.tb_console.Location = new System.Drawing.Point(0, 24);
            this.tb_console.Multiline = true;
            this.tb_console.Name = "tb_console";
            this.tb_console.ReadOnly = true;
            this.tb_console.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
            this.tb_console.Size = new System.Drawing.Size(632, 400);
            this.tb_console.TabIndex = 5;
            // 
            // MainTest
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(632, 446);
            this.Controls.Add(this.tb_console);
            this.Controls.Add(this.statusStrip1);
            this.Controls.Add(this.menuStrip1);
            this.MainMenuStrip = this.menuStrip1;
            this.Name = "MainTest";
            this.Text = "JSBSimNET Test App";
            this.FormClosing += new System.Windows.Forms.FormClosingEventHandler(this.MainTest_FormClosing);
            this.menuStrip1.ResumeLayout(false);
            this.menuStrip1.PerformLayout();
            this.statusStrip1.ResumeLayout(false);
            this.statusStrip1.PerformLayout();
            this.ResumeLayout(false);
            this.PerformLayout();

        }
        #endregion

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }

        /// <summary>
        /// Eventhandler for form closing
        /// </summary>
        private void MainTest_FormClosing(object sender, FormClosingEventArgs e)
        {
            jsbsimworker.CancelAsync();
        }

        /// <summary>
        /// Eventhandler for "Cancel" button click. Requests that JSBSimWorker stop ASAP
        /// </summary>
        private void cancelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            jsbsimworker.CancelAsync();
        }

        /// <summary>
        /// Eventhandler for JSBSimWorker WorkerCompleted
        /// </summary>
        private void WorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Cancelled)
            {
                Trace.WriteLine(Environment.NewLine + "Simulation cancelled");
            }
            else if (e.Error != null)
            {
                Trace.WriteLine(Environment.NewLine + "Exception: " + e.Error.ToString());
            }
            else
            {
                Trace.WriteLine(Environment.NewLine + "Complete: " + e.Result); // from DoWork
            }
            setStatus(false);
        }

        /// <summary>
        /// Updates GUI to reflect "running" vs. "ready" status
        /// </summary>
        private void setStatus(bool running)
        {
            this.ToolStripMenuItem_Load.Enabled = !running;
            this.ToolStripMenuItem_Cancel.Enabled = running;
            if (running)
            {
                this.toolStripStatusLabel1.Text = "Running...";
                tb_console.Clear();
            }
            else
            {
                this.toolStripStatusLabel1.Text = "Ready";
            }
        }

        /// <summary>
        /// Event handler for "Load" menu opening. Populates the "Models" and "Scripts" sub-menus
        /// </summary>
        private void ToolStripMenuItem_Load_DropDownOpening(object sender, EventArgs e)
        {
            try
            {
                populateToolStripMenuItem_LoadModels(this.ToolStripMenuItem_LoadModels);
                populateToolStripMenuItem_LoadScripts(this.ToolStripMenuItem_LoadScripts);
            }
            catch (DirectoryNotFoundException ex)
            {
                MessageBox.Show(ex.Message, "Directory not found");
            }
        }

        /// <summary>
        /// Populates the given ToolStripMenuItem with models / reset files
        /// </summary>
        private void populateToolStripMenuItem_LoadModels(ToolStripMenuItem tsmi)
        {
            //wipe it
            tsmi.DropDown.Items.Clear();

            // get an array of all sub-directories in the aircraft directory (aircraft)
            String aircraftDirectory = rootDirectory + "\\aircraft\\";
            string[] aircraftDirs = Directory.GetDirectories(aircraftDirectory);

            // for each aircraft, make a menu item
            foreach (string aircraftDir in aircraftDirs)
            {
                String aircraftName = aircraftDir.Replace(aircraftDirectory, "");
                ToolStripMenuItem aircraftTSI = new ToolStripMenuItem(aircraftName);
                tsmi.DropDownItems.Add(aircraftTSI);

                // attach an event handler
                aircraftTSI.DropDown.ItemClicked +=
                        new ToolStripItemClickedEventHandler(LoadModels_ItemClicked);

                // get an array of all "reset" files
                string[] resetFiles = Directory.GetFiles(aircraftDir, "reset??.xml");

                //for each reset file, make a menu item
                foreach (string resetFile in resetFiles)
                {
                    String resetName = resetFile.Replace(aircraftDirectory, "");
                    resetName = resetName.Substring(resetName.LastIndexOf("\\") + 1, 7);
                    ToolStripMenuItem resetTSI = new ToolStripMenuItem(resetName);
                    aircraftTSI.DropDownItems.Add(resetTSI);
                }
            }
        }

        /// <summary>
        /// Populates the given ToolStripMenuItem with script files
        /// </summary>
        private void populateToolStripMenuItem_LoadScripts(ToolStripMenuItem tsmi)
        {
            //wipe it
            tsmi.DropDown.Items.Clear();

            // get an array of all .xml files in the scripts directory
            String scriptsDirectory = rootDirectory + "\\scripts\\";
            string[] scriptsDirs = Directory.GetFiles(scriptsDirectory, "*.xml");

            // attach an event handler
            tsmi.DropDown.ItemClicked +=
                    new ToolStripItemClickedEventHandler(LoadScripts_ItemClicked);

            // for each script file, make a menu item
            foreach (string scriptsDir in scriptsDirs)
            {
                String scriptName = scriptsDir.Replace(scriptsDirectory, "");
                //scriptName = scriptName.Remove(scriptName.LastIndexOf("."));
                ToolStripMenuItem scriptTSI = new ToolStripMenuItem(scriptName);
                tsmi.DropDownItems.Add(scriptTSI);
            }
        }

        /// <summary>
        /// Event handler for LoadModels menu clicked. Loads and runs the model / reset file pair the user selected (in a separate thread)
        /// </summary>
        private void LoadModels_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            string modelFileName = e.ClickedItem.OwnerItem.Text.ToString();
            string resetFileName = e.ClickedItem.Text.ToString();
            
            //If we aren't already working (we shouldn't be), then run the simulation in a thread
            if (!jsbsimworker.IsBusy) jsbsimworker.RunWorkerAsync(new Object[] { modelFileName, resetFileName });

            // put GUI in "running" status
            setStatus(true);
        }

        /// <summary>
        /// Event handler for LoadScripts menu clicked. Loads and runs the script file the user selected (in a separate thread)
        /// </summary>
        private void LoadScripts_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            string scriptFileName = e.ClickedItem.Text.ToString();

            //If we aren't already working (we shouldn't be), then run the simulation in a thread
            if(!jsbsimworker.IsBusy) jsbsimworker.RunWorkerAsync(new Object[] { scriptFileName });

            // put GUI in "running" status
            setStatus(true);
        }
    }
}
